/* 
 * splinee_data.h -- ePiX parametric spline functions
 *
 * This file is part of ePiX, a C++ library for creating high-quality 
 * figures in LaTeX 
 *
 * Version 1.1.4
 * Last Change: June 18, 2007
 *
 * 
 * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007
 * Andrew D. Hwang <rot 13 nujnat at zngupf dot ubylpebff dot rqh>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 *
 *
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
#ifndef EPIX_SPLINE_DATA
#define EPIX_SPLINE_DATA

namespace ePiX {

  template <typename T>
    T spl_pt(const T& p1, const T& p2, const T& p3, double t)
    {
      return (1-t)*(1-t)*p1 + 2*t*(1-t)*p2 + t*t*p3;
    }

  template <typename T>
    T spl_pt(const T& p1, const T& p2, const T& p3, const T& p4, double t)
    {
      const double s(1-t);
      return s*s*(s*p1 + 3*t*p2) + t*t*(3*s*p3 + t*p4);
    }
} // end of namespace

#endif /* EPIX_SPLINE_DATA */
